//
//  ASElementArray.m
//  Paste2
//
//  Created by Andrew Finnell on 10/28/09.
//  Copyright 2009 Order N Development, LLC. All rights reserved.
//

#import "ASElementArray.h"
#import "ASObject.h"

extern id ObjectFromReply(AppleEvent reply);

@implementation ASElementArray

- (id) initWithContainer:(ASObject *)container key:(NSString *)key
{
	self = [super init];
	
	if ( self != nil ) {
		_container = [container retain];
		_key = [key copy];
	}
	
	return self;
}

- (void) dealloc
{
	[_container release];
	[_key release];
	
	[super dealloc];
}

- (NSAppleEventDescriptor *) insertionLocationForIndex:(NSInteger)index
{
	NSScriptObjectSpecifier *indexSpecifier = [[[NSIndexSpecifier alloc] initWithContainerClassDescription:[_container scriptClassDescription] 
													 containerSpecifier:[_container objectSpecifier] 
																	key:_key 
																  index:index] autorelease];

	AEDesc desc = {};
	AEBuildError error = {};
	AEBuildDesc(&desc, &error, "insl{kobj:@,kpos:enum('befo')}", [[indexSpecifier descriptor] aeDesc]);
		
	return [[[NSAppleEventDescriptor alloc] initWithAEDescNoCopy:&desc] autorelease];
}

- (ASObject *) objectAtIndex:(NSUInteger)index
{
	NSScriptObjectSpecifier *indexSpecifier = [[[NSIndexSpecifier alloc] initWithContainerClassDescription:[_container scriptClassDescription] 
																						containerSpecifier:[_container objectSpecifier] 
																									   key:_key 
																									 index:index] autorelease];

	return [[[ASObject alloc] initWithScriptObjectSpecifier:indexSpecifier] autorelease];
}

- (NSUInteger) count
{
	FourCharCode elementCode = [[_container scriptClassDescription] appleEventCodeForKey:_key];
	NSNumber *count = [_container invokeCommand:@"count" with:[NSNumber numberWithUnsignedInt:elementCode], @"ObjectClass", nil];
	return [count unsignedIntegerValue];
}

- (ASObject *) all
{
	// Build the descriptor for "all"
	FourCharCode elementCode = [[_container scriptClassDescription] appleEventCodeForKey:_key];
	AEDesc desc = {};
	AEBuildError error = {};
	AEBuildDesc(&desc, &error, "'obj '{form:enum('indx'),want:type(@),seld:abso('all '),from:@}", 
				elementCode, [[[_container objectSpecifier] descriptor] aeDesc]);
	
	// Convert the descriptor into an ASObject
	NSAppleEventDescriptor *descriptor = [[[NSAppleEventDescriptor alloc] initWithAEDescNoCopy:&desc] autorelease];
	NSScriptObjectSpecifier *specifier = [NSScriptObjectSpecifier objectSpecifierWithDescriptor:descriptor];
	return [[[ASObject alloc] initWithScriptObjectSpecifier:specifier] autorelease];
}

@end
